package com.example.carillon_map_box

import android.app.Activity
import android.content.Context
import androidx.annotation.NonNull
import android.Manifest
import android.content.pm.PackageManager
import com.mapbox.geojson.Point

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.platform.PlatformViewRegistry
import java.util.*

/** CarillonMapBoxPlugin */
class CarillonMapBoxPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private var PERMISSION_REQUEST_CODE: Int = 367
  private var currentActivity: Activity? = null
  private var platformViewRegistry: PlatformViewRegistry? = null
  private var binaryMessenger: BinaryMessenger? = null
  private lateinit var channel : MethodChannel
  private lateinit var currentContext: Context

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    val messenger = flutterPluginBinding.binaryMessenger
    channel = MethodChannel(messenger, "carillon_map_box")
    channel.setMethodCallHandler(this)

    platformViewRegistry = flutterPluginBinding.platformViewRegistry
    binaryMessenger = messenger
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "startNavigation") {
      startNavigation(call, result)
    } else if (call.method == "sendAlert") {
      sendAlert(call, result)
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    currentActivity = null
    channel.setMethodCallHandler(null)
  }

  private fun checkPermission() {
    val hasperm = currentActivity?.checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION)
    if (hasperm != PackageManager.PERMISSION_GRANTED) {
      currentActivity?.requestPermissions(arrayOf(Manifest.permission.ACCESS_FINE_LOCATION), PERMISSION_REQUEST_CODE)
    }
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    currentActivity = binding.activity
    currentContext = binding.activity.applicationContext

    checkPermission()

    if (platformViewRegistry != null && binaryMessenger != null && currentActivity != null) {
      platformViewRegistry?.registerViewFactory("plugins/carillon_map_box",
              CarillonMapBoxFactory(binaryMessenger!!, currentActivity!!))
    }
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    currentActivity = binding.activity
  }

  override fun onDetachedFromActivity() {
    currentActivity!!.finish()
    currentActivity = null
  }

  override fun onDetachedFromActivityForConfigChanges() {
    TODO("Not yet implemented")
  }

  private fun startNavigation(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
    val options = call.arguments as Map<String, Objects>
    NavigationLauncher.startNavigation(currentActivity!!, options)
  }

  private fun sendAlert(@NonNull call: MethodCall, @NonNull result: MethodChannel.Result) {
    NavigationLauncher.sendAlert(currentActivity!!)
  }
}
