package com.example.carillon_map_box

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.pm.PackageManager
import android.view.View
import android.widget.Toast
import com.google.android.gms.location.FusedLocationProviderClient
import com.google.android.gms.location.LocationServices
import com.mapbox.geojson.Point
import com.mapbox.maps.CameraOptions
import com.mapbox.maps.MapInitOptions
import com.mapbox.maps.MapView
import com.mapbox.maps.Style
import com.mapbox.maps.plugin.animation.camera
import com.mapbox.maps.plugin.gestures.gestures
import com.mapbox.maps.plugin.locationcomponent.LocationProvider
import com.mapbox.maps.plugin.locationcomponent.OnIndicatorBearingChangedListener
import com.mapbox.maps.plugin.locationcomponent.OnIndicatorPositionChangedListener
import com.mapbox.maps.plugin.locationcomponent.location
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.platform.PlatformView

class CarillonMapBoxView(context: Context, messenger: BinaryMessenger, activity: Activity, id: Int,
                         creationParams: Map<String?, Any?>?) : PlatformView {
    private val _context = context
    private val _messenger = messenger
    private val _activity = activity
    private lateinit var _mapView: MapView
    private var PERMISSION_REQUEST_CODE: Int = 367
    private var _updatePos: Boolean = true

    private val onIndicatorPositionChangedListener = object : OnIndicatorPositionChangedListener {
        override fun onIndicatorPositionChanged(point: Point) {
            if (_updatePos) {
                _mapView.getMapboxMap().setCamera(CameraOptions.Builder().center(point).zoom(10.0).build())
                _updatePos = false
            }
        }
    }

    override fun getView(): View {
        return _mapView
    }

    override fun dispose() {}

    init {
        _mapView = MapView(_context)
        _mapView.getMapboxMap().loadStyleUri(Style.LIGHT,
                object : Style.OnStyleLoaded {
                    override fun onStyleLoaded(style: Style) {
                        _mapView.location.updateSettings {
                            enabled = true
                            pulsingEnabled = true
                        }
                    }
                }
        )

        _mapView.location.addOnIndicatorPositionChangedListener(onIndicatorPositionChangedListener)
    }
}